#!/bin/csh -fx
#
# Create the Livewire sample database
#


setenv db master

isql -U$1 -P$2 <<END

use master
go

if exists (select 1
            from  sysobjects
           where  name = 'customer'
            and   type = 'U')
   drop table customer
go
 

create table customer
  (
	name	varchar(50)	not null,
	ID	integer		not null,
	address	varchar(50) null,
	city	varchar(20) null,
	state	varchar(2) null,
	zip	varchar(10) null,
	phone	varchar(13) null
  )
go


create unique index cidx on customer (ID)
go


if exists (select 1
            from  sysobjects
           where  name = 'supplier'
            and   type = 'U')
   drop table supplier
go
 
create table supplier
  (
	name		varchar(50) not null,
	id              integer       not null,
	address         varchar(50) null,
	city            varchar(20) null,
	state           varchar(2) null,
	zip             varchar(10) null,
	phone           varchar(13) null
  )
go


create unique index sidx on supplier (id)
go

if exists (select 1
            from  sysobjects
           where  name = 'videos'
            and   type = 'U')
   drop table videos
go
 

create table videos
  (
	title           varchar(50) not null,
	id              integer not null,
	year            integer,
	category        varchar(25) null,
	quantity        integer not null,
	numonhand       integer not null,
	synopsis        text null
  )
go


create unique index vidx on videos (id)
go


if exists (select 1
            from  sysobjects
           where  name = 'rentals'
            and   type = 'U')
   drop table rentals
go
 
create table rentals
  (
	customerId	integer	not null,
	videoId		integer	not null,
	invoiceNum	integer	not null,
	rentalDate	datetime 	not null,
	dueDate		datetime not null,
	returnDate	datetime null
  )
go


create unique index ridx on rentals (customerId, videoId, invoiceNum)
go


if exists (select 1
            from  sysobjects
           where  name = 'purchase'
            and   type = 'U')
   drop table purchase
go
 
create table purchase
  (
	videoID		integer not null,
	supplierID	integer not null,
	orderNum	integer not null,
	unitPrice	decimal(10,2) not null,
	quantity	integer not null,
	purchDate	datetime not null
  )
go


create unique index pidx on purchase (videoID, supplierID, orderNum)
go


if exists (select 1
            from  sysobjects
           where  name = 'invoice'
            and   type = 'U')
   drop table invoice
go
 
create table invoice
  (
	lastnumber integer not null
  )
go


create unique index iidx on invoice (lastnumber)
go

if exists (select 1
            from  sysobjects
           where  name = 'categories'
            and   type = 'U')
   drop table categories
go
create table categories
  (
	category	varchar(25)	not null
  )
go
create unique index catidx on categories (category)
go

grant all on purchase to public
go

grant all on rentals to public
go

grant all on customer to public
go

grant all on supplier to public
go

grant all on videos to public
go

grant all on invoice to public
go
grant all on categories to public
go
exit
END

bcp $db..purchase in sybs_purchase.unl  -c -t "|"  -U$1 -P$2 
bcp $db..rentals in sybs_rental.unl  -c -t "|"  -U$1 -P$2 
bcp $db..customer in sybs_customer.unl  -c -t "|"  -U$1 -P$2 
bcp $db..supplier in sybs_supplier.unl  -c -t "|"  -U$1 -P$2 
bcp $db..videos in sybs_video.unl  -c -t "|"  -U$1 -P$2 
bcp $db..categories in sybs_category.unl  -c -t "|"  -U$1 -P$2 
